﻿using MathNet.Numerics.LinearAlgebra;
using Microsoft.ML;
using Microsoft.ML.Data;
using PLplot;
using System;
using System.Collections.Generic;
using System.Linq;

class Program
{
    static void Main(string[] args)
    {
        MLContext ctx = new MLContext();

        IDataView data = ctx.Data.LoadFromTextFile<ClickData>("click.csv", separatorChar: ',', hasHeader: true);
        
        var xyList = ctx.Data.CreateEnumerable<ClickData>(data, true);
        double[] xData = xyList.Select(xy => (double)xy.X).ToArray();
        double[] yData = xyList.Select(xy => (double)xy.Y).ToArray();

        // y = theta0 + (theta1 * x)
        (double theta1, double theta0) = GetEquation(xData, yData);
        Console.WriteLine($"y = {theta0} + {theta1} * x");

        Func<double, double> func =
            (x) => theta0 + theta1 * x;

        double y1 = func(0);
        double y2 = func(300);

        DrawPlotChart(xData, yData, new double[] { 0, 300 }, new double[] { y1, y2 });

        Console.WriteLine(data.ToString());
    }

    private static (double theta1, double theta0) GetEquation(double[] xData, double[] yData)
    {
        Matrix<double> matA = CreateMatrix.DenseOfColumnMajor(xData.Count(), 1, xData);
        Vector<double> add1 = Vector<double>.Build.DenseOfArray(Enumerable.Repeat(1.0, xData.Count()).ToArray());
        Matrix<double> matAwith1 = matA.InsertColumn(1, add1);

        Console.WriteLine(matAwith1);
        Matrix<double> matB = CreateMatrix.DenseOfColumnMajor(yData.Count(), 1, yData);

        Matrix<double> pinvMatA = matAwith1.PseudoInverse();
        Matrix<double> matX = pinvMatA * matB;

        return (matX[0, 0], matX[1, 0]);
    }

    private static void DrawPlotChart(double[] xData, double[] yData, double [] ptX, double [] ptY)
    {
        string chartFileName = "click.svg";

        int xMin = 0;
        int yMin = 0;

        int xMax = (int)xData.Max() + 10;
        int yMax = (int)yData.Max() + 10;

        using (var pl = new PLStream())
        {
            pl.sdev("svg");
            pl.sfnam(chartFileName);
            pl.spal0("cmap0_alternate.pal");
            pl.init();

            pl.env(xMin, xMax, yMin, yMax, AxesScale.Independent, AxisBox.BoxTicksLabelsAxes);

            pl.lab("X", "Y", "Click");

            char code = Symbol.Bullet; // == 17;
            pl.col0(2); //Blue

            pl.poin(xData, yData, code);
            pl.line(ptX, ptY);

            pl.eop();

            pl.gver(out var verText);
        }
    }
}
